clear
%% Numerical solution to Fractional Diffusion Equation
%% with Dirichlet BC: u(0,t)=u(L,t)=0

%% Discretise in space and time
L = 3; % spatial domain [0,L]
N = 1000; % number of divisions in space
h = L/N; % stepsize in space
x = linspace(0,L,N+1); % N+1 nodes on [0,L]
D = 1; % diffusion coefficent
alpha = 1.8; % fractional power
T = 0.1; % final time
n = 100; % number of time steps
tau = T/n; % time step
% Delta function initial condition
u0 = zeros(N-1,1);  u0(N/2) = 1/h;

%% Matrix representation of Laplacian (-Delta)
A = 1/h^2*full(gallery('tridiag', N-1));
[P,Lam] = eig(A); % diagonalise A such that A=P*Lam*P^T
lambda = diag(Lam); % eigenvalues
scaling = (1+D*tau*lambda.^(alpha/2)); % for option 5

%% using discrete sine transform
% lam = 4/h^2 * sin(pi*(1:N-1)'/2/N).^2; % formula for eigenvalues under Zero Dirichlet BC.
% scaling2 = 1+D*tau*lam.^(alpha/2); % for option 6

%% computing un
un = u0;
figure; 
plot(x, [0 u0' 0], 'LineWidth', 2); xlabel('x');ylabel('u(x,t)');
title({'Numerical solution of Fractional Diffusion Equation', 'BC: zero Dirichlet, IC: \delta(x-L/2)'});

for j = 1:n % timesteping
    %un = (eye(size(A)) + D*tau*A^(alpha/2)) \ un; % option 1
    %un = (P*diag(1+D*tau*lambda.^(alpha/2))*P')\un; %option 2
    %un = P*diag(1./(1+D*tau*lambda.^(alpha/2)))*P'*un; % option 3
    %un = P*(diag(1./(1+D*tau*lambda.^(alpha/2)))*(P'*un)); %option 4
    un = P*((P'*un)./scaling); % option 5: best option if doing diagonalisation
%     un = dst(idst(un) ./ scaling2); %option 6: most efficient! No need to even form the matrix A!
    
    if mod(j,n/5)==0 % plot every n/5 time steps
        hold on
        handle = plot(x, [0 un' 0], 'LineWidth', 2);
        legend(handle,['T = ',num2str(j*tau),' with \alpha = ',num2str(alpha)]);
        ylim([0 3]);
        shg;
    end
end
%% Comparison with exact-in-time solution for Fractional Diffusion Equation
un_eit = expm(-A^(alpha/2)*D*T)*u0;
figure
h(1) = plot(x,[0 un' 0], 'LineWidth', 2);
hold on
h(2) = plot(x,[0 un_eit' 0],'r:','LineWidth', 2);
title(['\alpha = ',num2str(alpha)]);legend(h,'Numerical sol.','Exact-in-time sol.')
